package rag

import (
	"context"
	"github.com/mitchellh/mapstructure"
	ragApi "leafdev.top/leaf/rag/api/rag"
	"leafdev.top/leaf/rag/models"
)

func (Service) ListDocument(ctx context.Context, req *ragApi.ListDocumentRequest) (*ragApi.ListDocumentResponse, error) {
	document, err := documentLogic.ListDocument(ctx, int(req.LibraryId))
	if err != nil {
		return nil, err
	}

	var documentApiResponse ragApi.ListDocumentResponse
	for _, document := range document {
		var documentApiResponseItem = ragApi.Document{
			Id:        int64(document.ID),
			Name:      document.Name,
			UserId:    document.UserID,
			LibraryId: document.LibraryID,
			CreatedAt: document.CreatedAt.String(),
			UpdatedAt: document.UpdatedAt.String(),
		}
		documentApiResponse.Documents = append(documentApiResponse.Documents, &documentApiResponseItem)
	}

	return &documentApiResponse, nil
}

func (Service) CreateDocument(ctx context.Context, req *ragApi.CreateDocumentRequest) (*ragApi.CreateDocumentResponse, error) {
	document, err := documentLogic.CreateDocument(ctx, int(req.LibraryId), req.Name)
	if err != nil {
		return nil, err
	}

	var documentApiResponse ragApi.CreateDocumentResponse
	err = mapstructure.Decode(document, &documentApiResponse)
	if err != nil {
		return nil, err
	}

	return &documentApiResponse, nil
}

//func (Service) UpdateDocument(ctx context.Context, _ *ragApi.UpdateDocumentRequest) (*ragApi.UpdateDocumentResponse, error) {
//	panic("implement me")
//}

//func (Service) DeleteDocument(ctx context.Context, _ *ragApi.DeleteDocumentRequest) (*ragApi.DeleteDocumentResponse, error) {
//	panic("implement me")
//}

func (Service) CreateDocumentBlock(ctx context.Context, req *ragApi.CreateDocumentBlockRequest) (*ragApi.CreateDocumentBlockResponse, error) {

	var createDB = &models.CreateDocumentBlock{
		Order:   req.Order,
		Content: req.Content,
	}

	documentBlock, err := documentLogic.CreateDocumentBlock(ctx, int(req.DocumentId), createDB)
	if err != nil {
		return nil, err
	}

	var documentBlockApiResponse = ragApi.CreateDocumentBlockResponse{
		DocumentBlock: &ragApi.DocumentBlock{
			Id:         int64(documentBlock.ID),
			Content:    documentBlock.Content,
			DocumentId: documentBlock.DocumentID,
			CreatedAt:  documentBlock.CreatedAt.String(),
			UpdatedAt:  documentBlock.UpdatedAt.String(),
		},
	}

	return &documentBlockApiResponse, nil
}

func (Service) ListDocumentBlock(ctx context.Context, req *ragApi.ListDocumentBlockRequest) (*ragApi.ListDocumentBlockResponse, error) {
	panic("implement me")
}

func (Service) UpdateDocumentBlock(ctx context.Context, req *ragApi.UpdateDocumentBlockRequest) (*ragApi.UpdateDocumentBlockResponse, error) {
	panic("implement me")
}

func (Service) DeleteDocumentBlock(ctx context.Context, req *ragApi.DeleteDocumentBlockRequest) (*ragApi.DeleteDocumentBlockResponse, error) {
	panic("implement me")
}
